// Copyright (c) .NET Foundation. All rights reserved.
// Licensed under the Apache License, Version 2.0. See License.txt in the project root for license information.

using Microsoft.Data.Entity.FunctionalTests.TestModels.Inheritance;

namespace Microsoft.Data.Entity.FunctionalTests
{
    public abstract class InheritanceFixtureBase
    {
        public virtual void OnModelCreating(ModelBuilder modelBuilder)
        {
            // TODO: Do this with Code First when we can

            var model = modelBuilder.Model;

            var country = model.AddEntityType(typeof(Country));
            var countryIdProperty = country.AddProperty("Id", typeof(int));
            countryIdProperty.RequiresValueGenerator = true;
            var countryKey = country.SetPrimaryKey(countryIdProperty);
            country.AddProperty("Name", typeof(string));

            var animal = model.AddEntityType(typeof(Animal));
            var animalSpeciesProperty = animal.AddProperty("Species", typeof(string));
            animalSpeciesProperty.RequiresValueGenerator = true;
            var animalKey = animal.SetPrimaryKey(animalSpeciesProperty);
            animal.AddProperty("Name", typeof(string));
            var countryFk = animal.AddForeignKey(animal.AddProperty("CountryId", typeof(int)), countryKey, country);

            var bird = model.AddEntityType(typeof(Bird));
            bird.BaseType = animal;
            bird.AddProperty("IsFlightless", typeof(bool));

            var kiwi = model.AddEntityType(typeof(Kiwi));
            kiwi.BaseType = bird;
            kiwi.AddProperty("FoundOn", typeof(Island));

            var eagle = model.AddEntityType(typeof(Eagle));
            eagle.BaseType = bird;
            eagle.AddProperty("Group", typeof(EagleGroup));

            var plant = model.AddEntityType(typeof(Plant));
            var plantSpeciesProperty = plant.AddProperty("Species", typeof(string));
            plantSpeciesProperty.RequiresValueGenerator = true;
            var plantKey = plant.SetPrimaryKey(plantSpeciesProperty);
            plant.AddProperty("Name", typeof(string));

            var flower = model.AddEntityType(typeof(Flower));
            flower.BaseType = plant;

            var rose = model.AddEntityType(typeof(Rose));
            rose.BaseType = flower;
            rose.AddProperty("HasThorns", typeof(bool));

            var daisy = model.AddEntityType(typeof(Daisy));
            daisy.BaseType = flower;

            var eagleFk = bird.AddForeignKey(bird.AddProperty("EagleId", typeof(string)), animalKey, eagle);

            country.AddNavigation("Animals", countryFk, false);
            eagle.AddNavigation("Prey", eagleFk, false);
        }

        public abstract InheritanceContext CreateContext();

        protected void SeedData(InheritanceContext context)
        {
            var kiwi = new Kiwi
                {
                    Species = "Apteryx haastii",
                    Name = "Great spotted kiwi",
                    IsFlightless = true,
                    FoundOn = Island.South
                };

            var eagle = new Eagle
                {
                    Species = "Aquila chrysaetos canadensis",
                    Name = "American golden eagle",
                    Group = EagleGroup.Booted
                };

            eagle.Prey.Add(kiwi);

            var rose = new Rose
                {
                    Species = "Rosa canina",
                    Name = "Dog-rose",
                    HasThorns = true
                };

            var daisy = new Daisy
                {
                    Species = "Bellis perennis",
                    Name = "Common daisy"
                };

            var nz = new Country { Id = 1, Name = "New Zealand" };

            nz.Animals.Add(kiwi);

            var usa = new Country { Id = 2, Name = "USA" };

            usa.Animals.Add(eagle);

            context.Set<Animal>().Add(kiwi);
            context.Set<Bird>().Add(eagle);
            context.Set<Country>().Add(nz);
            context.Set<Country>().Add(usa);
            context.Set<Rose>().Add(rose);
            context.Set<Daisy>().Add(daisy);

            context.SaveChanges();
        }
    }
}
